@@ -16,7 +16,7 @@ from lxml import etree
 from tempfile import NamedTemporaryFile
 
 from reportlab.graphics.shapes import (
-    _CLOSEPATH, _CURVETO, _LINETO, _MOVETO, Group, Path, Polygon, PolyLine, Rect,
+    _CLOSEPATH, _CURVETO, _LINETO, _MOVETO, Group, Path, Polygon, PolyLine, Rect, Line,
 )
 from reportlab.lib import colors
 from reportlab.lib.units import cm, inch
@@ -527,6 +527,28 @@ class TestStyleSheets:
         assert main_group.contents[0].contents[1].contents[0].fillColor == colors.black
         assert main_group.contents[0].contents[1].contents[0].strokeWidth == 1.5
 
+    def test_css_stylesheet_multiple_style_tags(self):
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
+            <?xml version="1.0"?>
+            <svg width="777" height="267" xml:space="preserve">
+              <defs>
+                <style type="text/css">
+                #p1 { fill:rgb(255,0,0) }
+                </style>
+                <style type="text/css">
+                #p2 { fill:rgb(0,0,0); }
+                </style>
+              </defs>
+              <g>
+                <path id="p1" d="M 0,-100 V 0 H 50"/>
+                <path id="p2" d="M 0,100 V 0 H 50"/>
+              </g>
+            </svg>
+        ''')))
+        main_group = drawing.contents[0]
+        assert main_group.contents[0].contents[0].contents[0].fillColor == colors.red
+        assert main_group.contents[0].contents[1].contents[0].fillColor == colors.black
+
 
 class TestGroupNode:
     def test_svg_groups_have_svgid(self):
@@ -688,6 +710,20 @@ class TestRectNode:
         assert rect.rx == 5
         assert rect.ry == 1.5
 
+    def test_rx_ry_lonely(self):
+        """If only rx or ry is present, the other default to the same value."""
+        converter = svglib.Svg2RlgShapeConverter(None)
+        node = svglib.NodeTracker(etree.XML(
+            '<rect rx="2.5" width="10" height="3" x="0" y="0"/>'
+        ))
+        rect = converter.convertRect(node)
+        assert rect.ry == 2.5
+        node = svglib.NodeTracker(etree.XML(
+            '<rect ry="1" width="10" height="3" x="0" y="0"/>'
+        ))
+        rect = converter.convertRect(node)
+        assert rect.rx == 1
+
     def test_strokewidth_0(self):
         """
         When strokeWidth = 0, we force strokeColor to None to avoid the stroke
@@ -702,6 +738,30 @@ class TestRectNode:
         assert rect.strokeColor is None
 
 
+class TestLineNode:
+    def test_length_zero(self):
+        converter = svglib.Svg2RlgShapeConverter(None)
+        node = svglib.NodeTracker(etree.XML(
+            '<line stroke="#000000" x1="10" y1="20" x2="10" y2="20" />'
+        ))
+        line = converter.convertLine(node)
+        assert isinstance(line, Line)
+
+        # Force a slight change in the first coordinate, so it's not invisible.
+        assert line.x1 != 10
+
+    def test_length_not_zero(self):
+        """ No change needed when length is not zero. """
+        converter = svglib.Svg2RlgShapeConverter(None)
+        node = svglib.NodeTracker(etree.XML(
+            '<line stroke="#000000" x1="10" y1="20" x2="10" y2="30" />'
+        ))
+        line = converter.convertLine(node)
+        assert isinstance(line, Line)
+
+        assert line.x1 == 10
+
+
 class TestPolylineNode:
     def test_filling(self):
         converter = svglib.Svg2RlgShapeConverter(None)
@@ -722,6 +782,50 @@ class TestPolylineNode:
         assert group.contents[0].fillColor == colors.white
         assert isinstance(group.contents[1], PolyLine)
 
+    def test_length_zero(self):
+        converter = svglib.Svg2RlgShapeConverter(None)
+        node = svglib.NodeTracker(etree.XML(
+            '<polyline fill="none" stroke="#000000" '
+            'points="10,50,10,50" />'
+        ))
+        polyline = converter.convertPolyline(node)
+        assert isinstance(polyline, PolyLine)
+
+        # Force a slight change in the first coordinate, so it's not invisible.
+        assert polyline.points[0] != 10
+
+    def test_odd_length(self):
+        converter = svglib.Svg2RlgShapeConverter(None)
+        node = svglib.NodeTracker(etree.XML(
+            '<polyline fill="none" stroke="#000000" '
+            'points="10,50,10,50,10" />'
+        ))
+        polyline = converter.convertPolyline(node)
+        assert polyline is None
+
+
+class TestPolygonNode:
+    def test_length_zero(self):
+        converter = svglib.Svg2RlgShapeConverter(None)
+        node = svglib.NodeTracker(etree.XML(
+            '<polygon fill="none" stroke="#000000" '
+            'points="10,50,10,50" />'
+        ))
+        polygon = converter.convertPolygon(node)
+        assert isinstance(polygon, Polygon)
+
+        # Force a slight change in the first coordinate, so it's not invisible.
+        assert polygon.points[0] != 10
+
+    def test_odd_length(self):
+        converter = svglib.Svg2RlgShapeConverter(None)
+        node = svglib.NodeTracker(etree.XML(
+            '<polygon fill="none" stroke="#000000" '
+            'points="10,50,10,50,10" />'
+        ))
+        polygon = converter.convertPolygon(node)
+        assert polygon is None
+
 
 class TestUseNode:
     def test_use(self, drawing_source=None):
@@ -750,6 +854,20 @@ class TestUseNode:
         assert isinstance(main_group.contents[2].contents[0], Rect)
         assert main_group.contents[2].contents[0].fillColor == colors.red
 
+    def test_use_href_svg2(self):
+        """In SVG 2, xlink:href="" can be simply href=""."""
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
+            <svg xmlns="http://www.w3.org/2000/svg"
+                 width="10cm" height="3cm" viewBox="0 0 100 30">
+              <defs>
+                <rect id="MyRect" x=".1" y=".1" width="99.8" height="29.8"
+                      fill="none" stroke="blue" />
+              </defs>
+              <use x="20" y="10" href="#MyRect" />
+            </svg>
+        ''')))
+        assert isinstance(drawing.contents[0].contents[0].contents[0], Rect)
+
     def test_use_with_defs_at_end(self):
         self.test_use(drawing_source='''\
             <?xml version="1.0"?>
@@ -848,6 +966,19 @@ class TestUseNode:
 
 
 class TestSymbolNode:
+    def test_symbol_unused(self):
+        """<symbol> by itself should not be rendered."""
+        drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
+            <?xml version="1.0"?>
+            <svg xmlns="http://www.w3.org/2000/svg"
+                 width="900" height="600" viewBox="0 0 100 100">
+                <symbol id="X">
+                    <path d="M 0,0100 V 0 H 50"/>
+                </symbol>
+            </svg>
+        ''')))
+        assert drawing.contents[0].contents == []
+
     def test_symbol_node(self):
         drawing = svglib.svg2rlg(io.StringIO(textwrap.dedent('''\
             <?xml version="1.0"?>
